;;; @file petload.s
;;; Commodore C2N tape loader for the C2N232 and the PET series
;;; @author Marko Mkel (msmakela@nic.funet.fi)

;;; Copyright  2002 Marko Mkel
;;;
;;;     This program is free software; you can redistribute it and/or modify
;;;     it under the terms of the GNU General Public License as published by
;;;     the Free Software Foundation; either version 2 of the License, or
;;;     (at your option) any later version.
;;;
;;;     This program is distributed in the hope that it will be useful,
;;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;     GNU General Public License for more details.
;;;
;;;     You should have received a copy of the GNU General Public License
;;;     along with this program; if not, write to the Free Software
;;;     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; PET BASIC 1
keylen	= $20d	; length in bytes
keybuf	= $20f	; data
bend	= $7c	; end of basic program text
fa	= $f1	; current device number
;;; PET BASIC 2 and later
keylen2	= $9e	; length in bytes
keybuf2	= $26f	; data
bend2	= $2a	; end of basic program text
fa2	= $d4	; current device number

stemp	= $ff	; placeholder for the byte being received
casbuf1	= $27a	; the start address of the cassette buffer #1
c2n1read = $e811	; PIA Control Register A
c2n2read = $e84d	; VIA Interrupt Flag Register
c2nwrite = $e840	; VIA output register B
c2nwrval = 8		; PB3 (cassette write)
;; lower or raise the cassette write line
#define c2nwr0 and #255-c2nwrval
#define c2nwr1 ora #c2nwrval

;; read a bit from CASS READ #1 to the Carry flag
#define c2n1rdbit lda c2n1read:cmp #$80:bit c2n1read-1
;; read a bit from CASS READ #2 to the Carry flag (same length as above)
#define c2n2rdbit lda c2n2read:and #$10:cmp #$10:nop
;; wait for high-to-low transition on CASS READ #1 and clear the status
#define c2n1rdlo .(:wait bit c2n1read:bpl wait:.):bit c2n1read-1
;; wait for high-to-low transition on CASS READ #2 (same length as above)
#define c2n2rdlo lda #$10:.(:wait bit c2n2read:beq wait:.):nop

	.text
	*=casbuf1
	.byte 1
	.word prg
	.word prgend
	.dsb 16, $20

cnt	.word 0		; number of bytes to transfer (negative)
	;; store the received data
store	sta $ffff
adr = store + 1
	inc adr
	bne nowrap
	inc adr+1
nowrap	inc cnt		; more bytes to transfer?
	bne recv
	inc cnt+1
	bne recv
	;; end of transfer => set the end address
	ldx adr
	lda adr+1
end1	stx bend
end2	sta bend+1
	;; copy the keyboard buffer
	ldx klen
	stx keylen
kcopy	lda keys-1,x
	dex
	sta keybuf,x
	bne kcopy
	;; return to BASIC
	cli
	rts

	;; transfer loop
recv	stx c2nwrite	; drop WRITE
p1t	c2n1rdlo
	sty c2nwrite	; raise WRITE
	lda #$7f
	sta stemp	; initialise the counter
	;; loop => receive bit, drop WRITE, receive bit, raise WRITE
recv01
p2t1	c2n1rdbit
	stx c2nwrite	; drop WRITE
	ror stemp
p2t2	c2n1rdbit
	sty c2nwrite	; raise WRITE
	ror stemp
	bcs recv01	; got all bits?
	lda stemp
	eor #$ff
	bcc store	; branch always

entry2 = *+192
	;; entry point for port #2
	ldx #klast-cnt		; transfer the code to cassette buffer #1
copy	lda cnt+191,x
	sta cnt-1,x
	dex
	bne copy
	ldx #7			; patch the code
patch	lda patch1,x
	sta p1t,x
	lda patch2,x
	sta p2t1,x
	sta p2t2,x
	dex
	bpl patch
	rts
	;; fall through
	;; entry point for port #1
entry	sei
	lda c2nwrite	; WRITE=1 already
	tay
	c2nwr0
	tax
	clc
	bcc recv	; jump to the transfer loop

	;; patches for port #2
patch1	c2n2rdlo
patch1e
patch2	c2n2rdbit
patch2e

	.dsb casbuf1+182-*, 0
klen	.byte kend-keys
keys	.byte "RUN", $0d	; contents for the keyboard buffer
kend	.dsb casbuf1+192-*, 0
klast

	*=$401
prg	.word nextln
	.word 2002
	.byte $9e		; SYS
	.byte $30 + (start / 1000)
	.byte $30 + ((start - (start / 1000 * 1000)) / 100)
	.byte $30 + ((start - (start / 100 * 100)) / 10)
	.byte $30 + (start - (start / 10 * 10))
	.byte $3a		; :
	.byte $a2		; NEW
	.byte 0
nextln	.word 0
start	ldx fa
	dex
	beq port1
	dex
	beq port2
	;; not BASIC 1 => try again
	ldx fa2
	dex
	beq patch0
	jsr entry2
	;; patch the code for later PET models
patch0	ldx #bend2
	stx end1+1
	inx
	stx end2+1
	ldx #keylen2
	lda #0
	stx kcopy-2
	sta kcopy-1
	stx kcopy0-2
	sta kcopy0-1
	ldx #<keybuf2
	stx kcopy0+5
	stx kcopy+5
	bne port1	; branch always
port2	jsr entry2
port1	ldx #k0end-keys0
	stx keylen
kcopy0	lda keys0-1,x
	dex
	sta keybuf,x
	bne kcopy0
	rts
keys0	.byte "SYS"
	.byte $30 + (entry / 100)
	.byte $30 + ((entry - (entry / 100 * 100)) / 10)
	.byte $30 + (entry - (entry / 10 * 10))
	.byte 13
k0end
prgend
