/*
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *
 *  Copyright 1999-2001 Michael Klein <michael.klein@puffin.lb.shuttle.de>
*/

#ifdef SAVE_RCSID
static char *rcsid =
    "@(#) $Id: cbm.c,v 1.12 2001/11/28 18:51:57 michael Exp $";
#endif

#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "../include/opencbm.h"
#include "cbm_module.h"

static char *cbm_dev_name = "/dev/cbm";

const char *cbm_get_driver_name(int port)
{
    return cbm_dev_name;
}

int cbm_driver_open(CBM_FILE *f, int port)
{
    *f = open(cbm_dev_name, O_RDWR);
    return (*f < 0) ? -1 : 0; /* FIXME */
}

void cbm_driver_close(CBM_FILE f)
{
    if(f >= 0) {
        close(f);
    }
}

int cbm_raw_write(CBM_FILE f, const void *buf, size_t size)
{
    return write(f, buf, size);
}

int cbm_raw_read(CBM_FILE f, void *buf, size_t size)
{
    return read(f, buf, size);
}

int cbm_listen(CBM_FILE f, __u_char dev, __u_char secadr)
{
    return ioctl(f, CBMCTRL_LISTEN, (dev<<8) | secadr);
}

int cbm_talk(CBM_FILE f, __u_char dev, __u_char secadr)
{
    return ioctl(f, CBMCTRL_TALK, (dev<<8) | secadr);
}

int cbm_open(CBM_FILE f, __u_char dev, __u_char secadr, const void *fname, size_t len)
{
    int rv, l;

    rv = ioctl(f, CBMCTRL_OPEN, (dev<<8) | secadr);
    if(rv == 0 && fname != NULL) {
        l = (len > 0) ? len : strlen(fname);
        if(l > 0) {
            rv = cbm_raw_write(f, fname, l) != l;
        }
        cbm_unlisten(f);
    }
    return rv;
}

int cbm_close(CBM_FILE f, __u_char dev, __u_char secadr)
{
    return ioctl(f, CBMCTRL_CLOSE, (dev<<8) | secadr);
}

int cbm_unlisten(CBM_FILE f)
{
    return ioctl(f, CBMCTRL_UNLISTEN);
}

int cbm_untalk(CBM_FILE f)
{
    return ioctl(f, CBMCTRL_UNTALK);
}

int cbm_get_eoi(CBM_FILE f)
{
    return ioctl(f, CBMCTRL_GET_EOI);
}

int cbm_reset(CBM_FILE f)
{
    return ioctl(f, CBMCTRL_RESET);
}

__u_char cbm_pp_read(CBM_FILE f)
{
    return ioctl(f, CBMCTRL_PP_READ);
}

void cbm_pp_write(CBM_FILE f, __u_char c)
{
    ioctl(f, CBMCTRL_PP_WRITE, c);
}

int cbm_iec_poll(CBM_FILE f)
{
    return ioctl(f, CBMCTRL_IEC_POLL);
}

int cbm_iec_get(CBM_FILE f, int line)
{
    return (ioctl(f, CBMCTRL_IEC_POLL) & line) != 0;
}

void cbm_iec_set(CBM_FILE f, int line)
{
    ioctl(f, CBMCTRL_IEC_SET, line);
}

void cbm_iec_release(CBM_FILE f, int line)
{
    ioctl(f, CBMCTRL_IEC_RELEASE, line);
}

int cbm_iec_wait(CBM_FILE f, int line, int state)
{
    return ioctl(f, CBMCTRL_IEC_WAIT, (line<<8) | state);
}

int cbm_device_status(CBM_FILE f, int drv, void *buf,  size_t bufsize)
{
    strncpy(buf, "99, DRIVER ERROR,00,00\r", bufsize);
    if(cbm_talk(f, drv, 15) == 0) {
        int bytes_read = cbm_raw_read(f, buf, bufsize);
        if(bytes_read == bufsize) {
            bytes_read--;
        }
        ((__u_char*)buf)[bytes_read] = '\0';
        cbm_untalk(f);
    }
    return atoi(buf);
}

int cbm_exec_command(CBM_FILE f, int drv, const void *cmd, size_t len)
{
    int rv;
    rv = cbm_listen(f, drv, 15);
    if(rv == 0) {
        if(len == 0) {
            len = strlen(cmd);
        }
        rv = cbm_raw_write(f, cmd, len) != len;
        cbm_unlisten(f);
    }
    return rv;
}
