/*
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *
 *  Copyright 2001 Michael Klein <michael.klein@puffin.lb.shuttle.de>
*/

#ifdef SAVE_RCSID
static char *rcsid =
    "@(#) $Id: detect.c,v 1.1 2001/10/01 18:00:51 michael Exp $";
#endif

#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "../include/opencbm.h"


int cbm_identify(CBM_FILE f, int drv, enum cbm_device_type_e *t,
                 const char **type_str)
{
    const char cmd[] = { 'M', '-', 'R', 0x40, 0xff, 0x02 };
    unsigned char buf[2];
    int rv = -1;
    unsigned short magic;

    if( t ) *t = cbm_dt_unknown;
    if( type_str ) *type_str = "";

    if( cbm_exec_command( f, drv, cmd, sizeof(cmd) ) == 0 &&
        cbm_talk( f, drv, 15 ) == 0 )
    {
        if( cbm_raw_read( f, buf, 2 ) == 2 )
        {
            magic = buf[0] | (buf[1] << 8);
            if(t)
            {
                switch( magic )
                {
                    case 0xaaaa: /* 1540 or 1541                   */
                    case 0xf00f: /* 1541-II                        */
                    case 0xcd18: /* 1541C           (not verified) */
                    case 0x10ca: /* DolphinDOS 1541 (not verified) */
                    case 0x6f10: /* SpeedDOS 1541                  */
                    case 0x8085: /* JiffyDOS 1541   (not verified) */
                    case 0xaeea: /* 64'er DOS 1541  (not verified) */
                        *t = cbm_dt_cbm1541;
                        break;
                    case 0xfed7: /* 1570            (not verified) */
                        *t = cbm_dt_cbm1570;
                        break;
                    case 0x02ac: /* 1571                           */
                        *t = cbm_dt_cbm1571;
                        break;
                    case 0x01ba: /* 1581                           */
                        *t = cbm_dt_cbm1581;
                        break;
                    default:     /* unknown                        */
                        break;
                }
            }
            if(type_str)
            {
                switch( magic )
                {
                    case 0xaaaa: *type_str = "1540 or 1541"   ; break;
                    case 0xf00f: *type_str = "1541-II"        ; break;
                    case 0xcd18: *type_str = "1541C"          ; break;
                    case 0x10ca: *type_str = "DolphinDOS 1541"; break;
                    case 0x6f10: *type_str = "SpeedDOS 1541"  ; break;
                    case 0x8085: *type_str = "JiffyDOS 1541"  ; break;
                    case 0xaeea: *type_str = "64'er DOS 1541" ; break;
                    case 0xfed7: *type_str = "1570"           ; break;
                    case 0x02ac: *type_str = "1571"           ; break;
                    case 0x01ba: *type_str = "1581"           ; break;
                    default:     *type_str = "*unknown*"      ; break;
                }
            }
            rv = 0;
        }
        cbm_untalk( f );
    }
    return rv;
}
