/*
 * Copyright  1994-1996 Marko Mkel and Olaf Seibert
 * Original Linux and Commodore 64/128/Vic-20 version by Marko Mkel
 * Ported to the PET and the Amiga series by Olaf Seibert
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define BUFSIZE 256
#include "prtrans.h"
#include "prcommon.h"

#if !defined(__MAIN_C__)
#define main	main_prwfile
#endif

int main(int argc, char **argv);

int
main(int argc, char **argv)
{
  FILE *file = stdin;
  unsigned char device = 8, secaddr = 1, byte;
  unsigned driveraddr;
  char **parameters = argv;
  char *remotefile;
  char *localfile;
  static char prgname[128];
  char *prg = prgname; /* loader program file name */
  int specified_remotefile = 0;
  baseaddr = portaddr[DEFAULT_PORT];

  while (*++parameters && **parameters == '-') { /* check for options */
    if (parameters[0][1] == '-') { /* "--" ends options */
      parameters++;
      break;
    }

    switch (parameters[0][1]) {
    case 'd':
      device = strtoul (*++parameters, NULL, 0);

      if (device > 15) {
	fprintf (stderr, "%s: Illegal device number specified.\n", *argv);
	return 1;
      }

      break;

    case 'l':
      prg = *++parameters;
      break;

    case 'p':
      baseaddr = strtoul (*++parameters, NULL, 16);

      if (baseaddr > 3) {
	fprintf (stderr, "%s: The printer port number must be between 0 and 3.\n",
			 *argv);
	return 1;
      }

      baseaddr = portaddr[baseaddr];
      break;

    case 's':
      secaddr = strtoul (*++parameters, NULL, 0);

      if (secaddr > 15) {
	fprintf (stderr, "%s: Illegal secondary address specified.\n", *argv);
	return 1;
      }

      break;

    case '?':
    case 'h':
    Usage:
      fprintf (stderr, "%s: Writes a local file to a file on a remote "
		       "computer.\n\n", *argv);
      fprintf (stderr, "Usage: %s [options] filename [remote filename]\n",
		       *argv);
      fprintf (stderr, "Options:\n\t"
			 "-d device\t"
			     "Specify the device number (0-15)\n\t"
			 "-l filename\t"
			     "Specify the filename of the client program\n\t"
			 "-p port\t"
			     "Specify the printer port (0 to 3)\n\t"
			 "-s sec. addr.\t"
			     "Specify the secondary address (0-15)\n");
      fprintf (stderr, "The remote filename is converted to PETSCII.\n");

      return 1;

    default:
      fprintf (stderr, "%s: Illegal option `%s'.\n", *argv, *parameters);
      goto Usage;
    }
  }

  if (!*parameters) goto Usage;

  localfile = *parameters++;
  remotefile = strdup(localfile);

  if (*parameters) {
    remotefile = *parameters++;
    specified_remotefile = 1;
  }
  ascii2petscii(remotefile);

  if (!specified_remotefile && localfile[1] == ':') {
    localfile += 2;	/* strip 0: or 1: from local filename */
  }
  if (strcmp(localfile, "-") == 0) {
    file = stdin;
  } else if (!(file = fopen(localfile, "rb"))) {
    fprintf (stderr, "%s: Could not open the file `%s'.\n", *argv,
		       localfile);
    return 1;
  }

  if (prinit()) {
    fprintf (stderr, "%s: Could not get the I/O permissions.\n", *argv);
    return 1;
  }

  /* determine the program name to download */
  send_switch ();
  wait_output (REQ_INFO);
  receive_switch ();
  sprintf (prgname, "PRLINK_PRWFILE%u", wait_input());
  getname (prgname, prgname, "prwfile.prg");
  driveraddr = input();
  driveraddr |= input() << 8;
  input(); input(); /* skip the BASIC address information */

  {
    FILE *f = fopen(prg, "r");
    if (!f) {
      fprintf(stderr, "%s: Cannot find %s to download\n", *argv, prg);
      prclose();
      return 1;
    }

    if (prloadrun (driveraddr, f)) {
      fprintf (stderr, "%s: Could not upload and start %s\n", *argv, prg);
      prclose();
      return 6;
    }

    fclose(f);
  }

  receive_switch ();

  if (wait_input ()) {
    fprintf (stderr, "%s: not ready to transfer\n", *argv);
    prclose();
    return 6;
  }
  else
    fprintf (stderr, "%s: ok to transfer\n", *argv);

  send_switch();
  byte = strlen(remotefile);
  output(byte);
  psend(remotefile, byte);
  output(device);
  output(secaddr);

  receive_switch ();

  if ((byte = wait_input ()) != 0) {
    fprintf (stderr, "%s: file error %d\n", *argv, byte);
    send_switch ();
    output(255); /* terminate remote side */
    prclose();
    return 1;
  }
  fprintf(stderr, "Writing \"%s\" to \"%s\",%d,%d...\n",
          localfile, remotefile, device, secaddr);

  /* loop to send all file blocks */
  for (;;) {
    int length;
    int chksum;
    int i;

    length = fread(buffer + 1, 1, 255, file);

    chksum = 0;
    for (i = 0; i < length; i++) {
      chksum += buffer[1 + i];
    }
    buffer[0] = chksum;

    for (;;) {
      send_switch ();
      output((unsigned char)length);

      if (length == 0)
	break;	      /* end of file */

      psend(buffer, length + 1);

      receive_switch ();
      i = wait_input ();

      switch (i) {
      case 0x80:
	goto nextblock;
      case 0x81:
	fprintf(stderr, "checksum error, resending...\n");
	break;
      default:
	fprintf(stderr, "Aborting\n");
	goto abort;
      }
    }
  nextblock:
    if (length == 0)
      break;
  }
abort:

  if (file != stdin) fclose(file);

  send_switch ();
  output(255);	/* terminate remote side */

  prclose ();

  return 0;
}

