/*
 * Copyright  1994-1996 Marko Mkel and Olaf Seibert
 * Original Linux and Commodore 64/128/Vic-20 version by Marko Mkel
 * Ported to the PET and the Amiga series by Olaf Seibert
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#define BUFSIZE 65536
#include "prtrans.h"
#include "prcommon.h"

void getname(char *dest, char *env_file, char *def_file) {
    char *dir, *file;

    dir = getenv("PRLINKDIR");
    if (dir == NULL)
	dir = "";
    file = getenv(env_file);
    if (file == NULL)
	file = def_file;
    sprintf(dest, "%s%s", dir, file);
}

/*
 * only deal with letters in upper/lowercase character set.
 * a-z = 0x41 - 0x5A
 * A-Z = 0xC1 - 0xDA
 * Punctuation 0x20 - 0x3F and 0x5B - 0x5F is virtually identical.
 */

void ascii2petscii(unsigned char *p) {
  while (p[0]) {
    if (p[0] == '\n') {
      p[0] = 13;
    } else if (p[0] >= 'A' && p[0] <= 'Z') {
      p[0] += -'A' + 0x41 + 0x80;
    } else if (p[0] >= 'a' && p[0] <= 'z') {
      p[0] += -'a' + 0x41;
    }
    p++;
  }
}

void petscii2ascii(unsigned char *p) {
  while (p[0]) {
    if (p[0] == 13) {
      p[0] = '\n';
    } else if (p[0] >= 0xC1 && p[0] <= 0xDA) {
      p[0] -= -'A' + 0x41 + 0x80;
    } else if (p[0] >= 0x41 && p[0] <= 0x5A) {
      p[0] -= -'a' + 0x41;
    }
    p++;
  }
}

/* strip the base filename from a path name */

void basename(char **remotefile) {
  char *c,*n;
  if (remotefile && *remotefile) {
    for (c=*remotefile;*c;c++);
    for (;(*c!='/')&&(c>*remotefile);c--);
    if (*c=='/') {
      n=strdup(c+1);
      free (*remotefile);
      *remotefile=n;
    }
  }
}

int prloadrun(unsigned driveraddr, FILE *f) {
  unsigned start, length;

  if (!f || fseek(f, 0, SEEK_END))
    return -1; /* invalid file handle or non-regular file */

  length = ftell(f) - 2;
  if (length > 65535)
    return -2; /* not a program file */

  if (fseek(f, 0, SEEK_SET))
    return -1; /* the file cannot be rewinded */

  start = fgetc(f);
  start |= fgetc(f) << 8;

  if (length > fread (buffer, 1, length, f))
    return -1; /* the file could not be read */

  if (buffer[0] == 0x4c) { /* new-style client with jump table */
    unsigned i, addr;

    addr = buffer[1] | buffer[2] << 8;

    if (addr - start != 24)
      goto dontpatch; /* the jump table is wrong size */

    /* check the validity of the jump table */

    addr = buffer[4] | buffer[5] << 8; /* get server base address */

    for (i = 0; i < 21; i += 3)
      if (buffer[i + 3] != 0x4c ||
	  (buffer[i + 4] | buffer[i + 5] << 8) != addr + i)
	goto dontpatch;

    /* patch the jump table */
#if 0
    /* relocate the jumps in the local jump table */

    for (i = 0; i < 21; i += 3) {
      buffer[i + 4] = (unsigned char)(driveraddr + i);
      buffer[i + 5] = (unsigned char)((driveraddr + i) >> 8);
    }
#else
    /* bypass the jump table */

    /* copy the jump table of the prserver daemon */

    send_switch ();
    output (REQ_SAVE); /* op code for download */
    output (0); /* bank address */

    receive_switch ();
    if (input ())
      return -3; /* not ready to transfer */

    send_switch ();
    output ((unsigned char)driveraddr);
    output ((unsigned char)(driveraddr >> 8));
    output ((unsigned char)(driveraddr + 18));
    output ((unsigned char)((driveraddr + 18) >> 8));

    receive_switch ();
    preceive (&buffer[3], 18);
#endif

    /* bypass the jumps to the jump table */

    for (i = 0; i < length; i++) {
      if (buffer[i] == 0x20 || buffer[i] == 0x4c) {
	addr = (buffer[i+1] | (buffer[i+2] << 8)) - start - 3;

	if (addr < 21 && !(addr % 3)) { /* it was a jump to the jump table */
	  buffer[++i] = buffer[addr + 4]; /* replace the address */
	  buffer[++i] = buffer[addr + 5];
	}
      }
    }
  }

 dontpatch:

  send_switch ();
  output (REQ_LOAD); /* op code for upload */
  output (0); /* specify the bank address */
  receive_switch ();
  if (input ())
    return -3; /* not ready to transfer */

  send_switch ();
  output ((unsigned char)start);
  output ((unsigned char)(start >> 8));
  output ((unsigned char)(start + length));
  output ((unsigned char)((start + length) >> 8));

  psend (buffer, length);

  send_switch ();
  output (REQ_JUMP); /* op code for starting a program */
  output (0); /* the bank address */
  output ((unsigned char)start);
  output ((unsigned char)(start >> 8));

  return 0;
}
