/*
 * Copyright  1996-1997 Marko Mkel and Frank Kontros
 * Idea by Frank Kontros; ported to Linux and adapted to the
 * prlink package by Marko Mkel
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * This program contains a simple file server that responds to the requests
 * from the LOAD/SAVE client, which is implemented in prclient.a65.
 */

/* BUGS:
 *   - no support for disk directory
 *   - no PETSCII<->ASCII filename translation
 *   - the program won't work properly when loading/saving files
 *     longer than 65534 bytes
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define __MAIN_C__
#define __PRMAIN_C__
#define BUFSIZE 65536
#include "prtrans.h"

#undef P
#define P(func) func

struct cable {
  char   *name;
  int (*prinit) (void);
  void (*prclose) (void);
  unsigned (*input) (void);
  void (*output) (unsigned char byte);
#ifndef MSDOS
  unsigned (*wait_input) (void);
  void (*wait_output) (unsigned char byte);
#endif
  void (*send_switch) (void);
  void (*receive_switch) (void);
  void (*preceive) (unsigned char *buffer, unsigned length);
  void (*psend) (unsigned char *buffer, unsigned length);
};

struct cable *c;

#undef E
#undef P
#define E extern
#ifdef __amiga__
#  define P(func) func ## _pramiga88
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _pramiga48
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _pramigatn
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _pramieci
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _pramiec
#  include "prtransd.h"
#else
#  define P(func) func ## _prtrans9
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _prtrans8
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _prtrans4
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _prtrans2
#  include "prtransd.h"
#  undef P
#  define P(func) func ## _prtrans1
#  include "prtransd.h"
#endif
#undef P
#undef E
#define P(func) func

#ifndef MSDOS
#define CABLE(s) { #s, \
		   &prinit_ ## s, &prclose_ ## s, &input_ ## s, &output_ ##s, \
		   &wait_input_ ## s, &wait_output_ ## s, \
		   &send_switch_ ## s, &receive_switch_ ## s, \
		   &preceive_ ## s, &psend_ ## s }
#else
#define CABLE(s) { #s, \
		   &prinit_ ## s, &prclose_ ## s, &input_ ## s, &output_ ##s, \
		   &send_switch_ ## s, &receive_switch_ ## s, \
		   &preceive_ ## s, &psend_ ## s }
#endif

struct cable clist[] = {
#ifdef __amiga__
  CABLE(pramiga88),
  CABLE(pramiga48),
  CABLE(pramigatn),
  CABLE(pramiec),
  CABLE(pramieci),
#else
  CABLE(prtrans9),
  CABLE(prtrans8),
  CABLE(prtrans4),
  CABLE(prtrans2),
  CABLE(prtrans1),
#endif
  { NULL, }
};

int findcable (char *name);
int main (int argc, char **argv);
void mainloop ();
char *getname ();

int findcable (char *name)
{
  for (c = clist; c->name; c++)
#if 1
    if (strstr(c->name, name)) {
#else
    if (strcmp(c->name, name) == 0) {
#endif
      prinit = c->prinit;
      prclose = c->prclose;
      input = c->input;
      output = c->output;
      preceive = c->preceive;
      psend = c->psend;
      send_switch = c->send_switch;
      receive_switch = c->receive_switch;
#ifndef MSDOS
      wait_input = c->wait_input;
      wait_output = c->wait_output;
#endif
      return 0;
    }

  return -1;
}

int main(int argc, char **argv)
{
  char *myname = *argv;
  baseaddr = portaddr[DEFAULT_PORT];

  if (findcable (clist->name)) {
    fprintf(stderr,
	    "%s: The protocol could not be set.\n", myname);
    return -1;
  }

  /* Process the command line options */
  for (;;) {
    if (argc > 1 && !strcmp(argv[1], "-c")) { /* -c option: specify the cable */
      if (argc < 3 || findcable(argv[2])) {
	fprintf(stderr,
		"%s: Invalid cable specified.  Available options are:\n",
		myname);
	for (c = clist; c->name; c++)
	  fprintf(stderr, "\t%s\n", c->name);

	return -1;
      }
      argv[2] = *argv;
      argc -= 2;
      argv += 2;
      continue;
    }

    if (argc > 1 && !strcmp(argv[1], "-p")) { /* -p option: specify the port */
      if (argc < 3 || ((baseaddr = atoi (argv[2])) > 3)) {
	fprintf(stderr,
		"%s: The port address must be between 0 and 3.\n",
		myname);

	return -1;
      }
      baseaddr = portaddr[baseaddr];
      argv[2] = *argv;
      argc -= 2;
      argv += 2;
      continue;
    }

    break;
  }

  if (prinit ()) {
    fprintf (stderr, "%s: Could not get the I/O permissions.\n", *argv);
    return -2;
  }

  mainloop();
  return 0;
}

void mainloop()
{
  enum command { CMD_LOAD = 1, CMD_SAVE } cmd;

  char *filename;
  FILE *f;
  int len;

  for (;;) {
    receive_switch();
    cmd = wait_input();

    switch (cmd) {
    case CMD_LOAD:
      filename = getname();
      if (!filename) {
	printf ("couldn't get filename\n");
	return;
      }
      fprintf (stderr, "loading %s...\n", filename);

      f = fopen (filename, "rb");
      free (filename);
      if (f) {
	len = fread (buffer, 1, BUFSIZE, f);
	fclose (f);
      }
      else
	len = 0;

      send_switch();
      if (len < 2) {
	output(1); /* send negative status code */
      }
      else {
	output(0); /* send positive status code and the file */
	output(len - 2);
	output((len - 2) >> 8);
	psend(buffer, len);
      }
      break;
    case CMD_SAVE:
      filename = getname();
      if (!filename) {
	fprintf (stderr, "fatal: couldn't get filename\n");
	return;
      }
      fprintf (stderr, "saving %s...\n", filename);
      f = fopen (filename, "wb");
      free (filename);

      send_switch();
      if (!f) {
	output(1); /* send negative status code */
      }
      else {
	int l;
	output(0); /* send positive status code */
	receive_switch();
	len = input();
	len |= input() << 8;
	len += 2;
	preceive(buffer, len);
	l = fwrite (buffer, 1, len, f);
	fclose (f);

	send_switch();
	output(l != len); /* report status */
      }
      break;
    default:
      fprintf (stderr, "fatal: caught undefined command %u\n", cmd);
      return;
    }
  }
}

char *getname()
{
  unsigned len;
  char *string;

  len = input();

  if (!len || !(string = malloc (len + 1)))
    return NULL; /* couldn't get the memory */

  /* Get the string and add the null termination */
  preceive (string, len);
  string[len] = 0;
  return string;
}
