/*
 * Copyright  1994-1996 Marko Mkel and Olaf Seibert
 * Original Linux and Commodore 64/128/Vic-20 version by Marko Mkel
 * Ported to the PET and the Amiga series by Olaf Seibert
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __linux__
#include <signal.h>
#include <unistd.h>
#include <asm/io.h>
#endif /* __linux__ */

#ifdef __FreeBSD__
#include <signal.h>
#include <unistd.h>
#include <machine/cpufunc.h>
#endif /* __FreeBSD__ */

#define __PRTRANS2_C__
#include "prtrans.h"

static void cleanup (int i);

static void cleanup (int i) {
  outb(0, baseaddr); /* this ensures that the server doesn't remain blocked */
  exit(-1);
}

int prinit (void) {
  stataddr = baseaddr + 1;
#ifdef __linux__
  if (ioperm (baseaddr, 2, 1))
    return -1;

  seteuid(getuid()); /* give up the super user privileges */
  signal(SIGINT, cleanup);
  signal(SIGTERM, cleanup);
#endif /* __linux__ */
  outb(0, baseaddr);

  return 0;
}

void prclose (void) {
  if (baseaddr && !prinit())
    outb(0, baseaddr);
}

void output (unsigned char byte) {
  psend (&byte, 1);
}

#ifndef MSDOS
void wait_output (unsigned char byte) {
  outb(byte | 8, baseaddr);
  while(0x20 & inb(stataddr))
    usleep (SLEEP_TIME);

  outb(byte << 4, baseaddr);
  while(!(0x20 & inb(stataddr)));
}
#endif /* !defined(MSDOS) */

unsigned input (void) {
  unsigned char byte;

  preceive (&byte, 1);
  return byte;
}

#ifndef MSDOS
unsigned wait_input (void) {
  register unsigned char data;

  outb(0xf0, baseaddr);
  while(0x20 & inb(stataddr))
    usleep (SLEEP_TIME);

  outb(0, baseaddr);

  while(!(0x20 & inb(stataddr)));
  data = (0xc0 & inb(stataddr)) >> 6;
  outb(8, baseaddr);

  while(0x20 & inb(stataddr));
  data |= (0xc0 & inb(stataddr)) >> 4;
  outb(0, baseaddr);

  while(!(0x20 & inb(stataddr)));
  data |= (0xc0 & inb(stataddr)) >> 2;
  outb(8, baseaddr);

  while(0x20 & inb(stataddr));
  data |= 0xc0 & inb(stataddr);
  outb(0, baseaddr);

  while(!(0x20 & inb(stataddr)));

  return data ^ 0xaa;
}
#endif /* !defined(MSDOS) */

void send_switch() {}
void receive_switch() {}

#if defined(GAS_ENABLED)
void psend (unsigned char *buffer, unsigned length) {
  asm ("movw baseaddr,%%dx
	movl %1,%%ecx
	movl %0,%%ebx

sloop:	movb (%%ebx),%%al
	orb $8,%%al
	outb %%al,%%dx

	incw %%dx
wait1:
        inb %%dx,%%al
        testb $32,%%al
        jne wait1

        movb %0,%%al
        salb $4,%%al
        decw %%dx
        outb %%al,%%dx

        incw %%dx
wait2:
        inb %%dx,%%al
        testb $32,%%al
        je wait2
	decw %%dx

	inc %%ebx
	loop sloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "al", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void psend (unsigned char *buffer, unsigned len) {
  __asm {
    push ds
    mov dx,baseaddr
    mov cx,len
    lds bx,buffer
sloop:
    mov al,[bx]
    or al,8
    out dx,al

    inc dx
wait1:
    in al,dx
    test al,32
    je nwait1
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz wait1
    jz abort

nwait1:
    mov al,[bx]
# ifdef USE286
    shl al,4
# else
    shl al,1
    shl al,1
    shl al,1
    shl al,1
# endif
    dec dx
    out dx,al

    inc dx
wait2:
    in al,dx
    test al,32
    jne nwait2
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz wait1
    jz abort
nwait2:
    dec dx

    inc bx
    loop sloop
abort:
    pop ds
  }

  if (inb(0x60) == 1) cleanup(-1);
}
#else
void psend (unsigned char *buffer, unsigned length) {
  while (length--) {
    outb(*buffer | 8, baseaddr);
    while(0x20 & inb(stataddr));
    outb(*buffer++ << 4, baseaddr);
    while(!(0x20 & inb(stataddr)));
  }
}
#endif

#if defined(GAS_ENABLED)
void preceive (unsigned char *buffer, unsigned length) {
  asm ("movw baseaddr,%%dx
	movl %1,%%ecx
	movl %0,%%ebx

rloop:	movb $0xf0,%%al
	outb %%al,%%dx
	incw %%dx
input0:
	inb %%dx,%%al
	testb $32,%%al
	jne input0
	decw %%dx

	xorb %%al,%%al
	outb %%al,%%dx
	incw %%dx

input1:
	inb %%dx,%%al
	testb $32,%%al
	je input1
	decw %%dx
	andb $0xc0,%%al
	shrb $6,%%al
	mov %%al,(%%ebx)

	movb $8,%%al
	outb %%al,%%dx
	incw %%dx

input2:
	inb %%dx,%%al
	testb $32,%%al
	jne input2
	decw %%dx
	andb $0xc0,%%al
	shrb $4,%%al
	orb %%al,(%%ebx)

	xorb %%al,%%al
	outb %%al,%%dx
	incw %%dx

input3:
	inb %%dx,%%al
	testb $32,%%al
	je input3
	decw %%dx
	andb $0xc0,%%al
	shrb $2,%%al
	orb %%al,(%%ebx)

	movb $8,%%al
	outb %%al,%%dx
	incw %%dx

input4:
	inb %%dx,%%al
	testb $32,%%al
	jne input4
	decw %%dx
	andb $0xc0,%%al
	xorb $0xaa,%%al
	xorb %%al,(%%ebx)

	xorb %%al,%%al
	outb %%al,%%dx
	incw %%dx

input5:
	inb %%dx,%%al
	testb $32,%%al
	je input5

	decw %%dx

	inc %%ebx
	loop rloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void preceive (unsigned char *buffer, unsigned len) {
  __asm {
    push ds
    mov dx,baseaddr
    mov cx,len
    lds bx,buffer

rloop:
    mov al,0xf0
    out dx,al
    inc dx
input0:
    in al,dx
    test al,32
    je ninput0
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz input0
    jmp abort

ninput0:
    dec dx

    xor al,al
    out dx,al
    inc dx

input1:
    in al,dx
    test al,32
    jne ninput1
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz input1
    jz abort
ninput1:
    dec dx
    and al,0xc0
# ifdef USE286
    shr al,6
# else
    shr al,1
    shr al,1
    shr al,1
    shr al,1
    shr al,1
    shr al,1
# endif
    mov [bx],al

    mov al,8
    out dx,al
    inc dx

input2:
    in al,dx
    test al,32
    je ninput2
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz input2
    jz abort
ninput2:
    dec dx
    and al,0xc0
# ifdef USE286
    shr al,4
# else
    shr al,1
    shr al,1
    shr al,1
    shr al,1
# endif
    or [bx],al

    xor al,al
    out dx,al
    inc dx

input3:
    in al,dx
    test al,32
    jne ninput3
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz input3
    jz abort
ninput3:
    dec dx
    and al,0xc0
# ifdef USE286
    shr al,2
# else
    shr al,1
    shr al,1
# endif
    or [bx],al

    mov al,8
    out dx,al
    inc dx

input4:
    in al,dx
    test al,32
    je ninput4
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz input4
    jz abort
ninput4:
    dec dx
    and al,0xc0
    xor al,0xaa
    xor [bx],al

    xor al,al
    out dx,al
    inc dx

input5:
    in al,dx
    test al,32
    jne ninput5
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz input5
    jz abort
rlop:
    jmp rloop	; otherwise the branch would be out of range
ninput5:
    dec dx

    inc bx
    loop rlop	; otherwise the branch would be out of range
abort:
    pop ds
  }

  if (inb(0x60) == 1) cleanup(-1);
}
#else
void preceive (unsigned char *buffer, unsigned length) {
  register unsigned char data;

  while (length--) {
    outb(0xf0, baseaddr);
    while(0x20 & inb(stataddr));
    outb(0, baseaddr);

    while(!(0x20 & inb(stataddr)));
    data = (0xc0 & inb(stataddr)) >> 6;
    outb(8, baseaddr);

    while(0x20 & inb(stataddr));
    data |= (0xc0 & inb(stataddr)) >> 4;
    outb(0, baseaddr);

    while(!(0x20 & inb(stataddr)));
    data |= (0xc0 & inb(stataddr)) >> 2;
    outb(8, baseaddr);

    while(0x20 & inb(stataddr));
    data |= 0xc0 & inb(stataddr);
    outb(0, baseaddr);

    *buffer++ = data ^ 0xaa;

    while(!(0x20 & inb(stataddr)));
  }
}
#endif
