/*
 * Copyright  1994-1997 Marko Mkel and Olaf Seibert
 * Original Linux and Commodore 64/128/Vic-20 version by Marko Mkel
 * Ported to the PET and the Amiga series by Olaf Seibert
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __linux__
#include <unistd.h>
#include <asm/io.h>
#endif /* __linux__ */

#ifdef __FreeBSD__
#include <unistd.h>
#include <machine/cpufunc.h>
#endif /* __FreeBSD__ */

#define __PRTRANS9_C__
#include "prtrans.h"

int prinit (void) {
#ifdef __linux__
  if (ioperm (baseaddr, 3, 1))
    return -1;

  seteuid(getuid()); /* give up the super user privileges */
#endif /* __linux__ */

  stataddr = baseaddr + 1;
  ctrladdr = baseaddr + 2;
  outb(0, ctrladdr); /* A/B=1, STROBE=1 */

  return 0;
}

void prclose (void) {
  outb(0, ctrladdr);
}

void output (unsigned char byte) {
  psend (&byte, 1);
}

#ifndef MSDOS
void wait_output (unsigned char byte) {
  outb(byte, baseaddr);
  outb(9, ctrladdr); /* STROBE=0, SEL=0 (change port direction) */
  outb(8, ctrladdr); /* STROBE=1 */
  while (!(0x80 & inb(stataddr)))
    usleep (SLEEP_TIME); /* wait for BUSY to change */
}
#endif

unsigned input (void) {
  unsigned char byte;

  preceive (&byte, 1);
  return byte;
}

#ifndef MSDOS
unsigned wait_input (void) {
  register unsigned char data1, data;

  /* get 5 bits (higher 4 bits) and wait for handshaking */
  while (!(0x80 & (data1 = inb(stataddr)))) /* wait for -BUSY to change */
    usleep (SLEEP_TIME); /* wait before retrying */

  outb(2, ctrladdr); /* -STROBE=1, A/B=0 (lower bits from LS257) */
  data = (inb(stataddr) >> 3) & 0x0f; /* get lower 4 bits */
  data |= (data1 << 1) & 0xf0; /* combine the bits */
  outb(1, ctrladdr); /* -STROBE=0 */
  outb(0, ctrladdr); /* -STROBE=1 */

  return data;
}
#endif

void send_switch()
{
  while (!(inb(stataddr) & 0x80))
    usleep(SLEEP_TIME);
}
void receive_switch()
{
  outb(1, ctrladdr); /* -STROBE=0 */
  outb(0, ctrladdr); /* -STROBE=1 */
}

#if defined(GAS_ENABLED)
void psend (unsigned char *buffer, unsigned length) {
  asm ("movw baseaddr,%%dx
	movl %1,%%ecx
	movl %0,%%ebx

sloop:	movb (%%ebx),%%al
	outb %%al,%%dx
	addw $2,%%dx
	movb $9,%%al
	outb %%al,%%dx
	decb %%al
	outb %%al,%%dx
	decw %%dx

owait:	inb %%dx,%%al
	rolb $1,%%al
	jnc owait
	decw %%dx

	inc %%ebx
	loop sloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void psend (unsigned char *buffer, unsigned len) {
  __asm {
    mov dx,baseaddr
    mov cx,len

    push ds
    lds bx,buffer

sloop:
    mov al,[bx]
    out dx,al
    add dx,2
    mov al,9
    out dx,al
    dec al
    out dx,al
    dec dx

owait:
    in al,dx
    rol al,1
    jc nowait
    in al,0x60 /* esc pressed? */
    dec al
    jne owait
    je abort
nowait:
    decw dx

    inc bx
    loop sloop

abort:
    pop ds
  }

  if (inb(0x60) == 1) exit(-1);
}
#else
void psend (unsigned char *buffer, unsigned length) {
  register unsigned char byte;

  while (length--) {
    byte = *buffer++;
    outb(byte, baseaddr);
    outb(9, ctrladdr); /* STROBE=0, SEL=0 (change port direction) */
    outb(8, ctrladdr); /* STROBE=1 */
    while (!(0x80 & inb(stataddr)));
  }
}
#endif

#if defined(GAS_ENABLED)
void preceive (unsigned char *buffer, unsigned length) {
  asm ("movw baseaddr,%%dx
	movl %1,%%ecx
	movl %0,%%ebx

	incw %%dx
rloop:	movb $2,%%ah
iwait:	inb %%dx,%%al
	rolb $1,%%al
	jnc iwait

	incw %%dx
	xchgb %%ah,%%al
	outb %%al,%%dx
	decw %%dx
	inb %%dx,%%al
	rorb $3,%%al
	orb $16,%%al
	andb %%ah,%%al
	movb %%al,(%%ebx)
	incw %%dx
	movb $1,%%al
	outb %%al,%%dx
	xorb %%al,%%al
	outb %%al,%%dx
	decw %%dx
        inc %%ebx
        loop rloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void preceive (unsigned char *buffer, unsigned len) {
  __asm {
    mov dx,baseaddr
    mov cx,len

    push ds
    lds bx,buffer

    inc dx
rloop:
    mov ah,2
iwait:
    in al,dx
    rol al,1
    jc niwait
    in al,0x60 /* esc pressed? */
    dec al
    jne iwait
    je abort

niwait:
    inc dx
    xchg al,ah
    out dx,al
    dec dx
    in al,dx
    ror al,3
    or al,16
    and al,ah
    mov [bx],al
    inc dx
    mov al,1
    out dx,al
    xor al,al
    out dx,al
    dec dx
    inc bx
    loop rloop

abort:
    pop ds
  }

  if (inb(0x60) == 1) exit(-1);
}
#else
void preceive (unsigned char *buffer, unsigned length) {
  register unsigned char data1, data;

  while (length--) {
    /* get 5 bits (higher 4 bits) and wait for handshaking */
    while (!(0x80 & (data1 = inb(stataddr)))); /* wait for -BUSY to change */

    outb(2, ctrladdr); /* A/B=0 (lower bits from LS257) */
    data = (inb(stataddr) >> 3) & 0x0f; /* get lower 4 bits */
    data |= (data1 << 1) & 0xf0; /* combine the bits */

    outb(1, ctrladdr); /* -STROBE=0 */
    outb(0, ctrladdr); /* -STROBE=1 */

    *buffer++ = data;
  }
}
#endif
