/* MakeAll.rexx 
 *
 * Make all combinations of binaries
 * Assumes GCC, GNU Make and DASM 2.02 with Olaf Seibert's modifications.
 */

/* Process arguments */

doit = 0
build8bit = 1
buildamiga = 1
copylibs = 1
doclean = 0

args = arg(1)
do i = 1 to words(args)
    select
    when word(args, i) = "doit" then doit = 1
    when word(args, i) = "no8bit" then build8bit = 0
    when word(args, i) = "noamiga" then buildamiga = 0
    when word(args, i) = "nolibs" then copylibs = 0
    when word(args, i) = "clean" then doclean = 1
    otherwise signal usage
    end
end


/* Set all options */

targets. = ""
cables.  = ""
ramexps. = ""
filexts. = ""

/* target types */
targets.0 = 6
targets.1 = "pet3001";	cables.1  = "prlink88 prlink transnib"
targets.2 = "pet4001";	cables.2  = "prlink88 prlink transnib"
targets.3 = "vic20";	cables.3  = "prlink88 prlink transnib iec"
targets.4 = "c64";	cables.4  = "prlink88 prlink transnib iec"
			ramexps.4 = "piaexp reuexp piaexp|reuexp"
			filexts.4 = "pia    reu    piareu"
targets.5 = "c128";	cables.5  = cables.4
			ramexps.5 = ramexps.4
			filexts.5 = filexts.4
targets.6 = "c16";	cables.6  = "iec"

call run 'assign base: /'
bindir = 'base:bin'

/* Clean up the destination for all our work */

if doclean then do
    call run 'rm -rf 'bindir
end

call run 'mkdir -p 'bindir

/* Make 6502 programs */

if build8bit then do

    /* Iterate through targets */
    do it = 1 to targets.0
	target = targets.it

	machdir = '/base/bin/'target
	call run 'mkdir -p 'machdir

	/* Iterate through the target's cables */
	do ic = 1 to words(cables.it)
	    cable = word(cables.it, ic)

	    call run 'rm -f *.prg'
	    call run 'bin:make TARGET='target '"RAMEXP=none" "CABLE='cable'" prprgs'
	    if ic == 1 then do
		call run 'cp -p *.prg 'machdir
	    end

	    call run 'mkdir -p 'machdir'/'cable
	    call run 'cp -p prserver.prg 'machdir'/'cable

	    /* Iterate through the target's ram expansions */
	    do ie = 1 to words(ramexps.it)
		ramexp = word(ramexps.it, ie)
		filext = word(filexts.it, ie)
		call run 'rm -f prserver.prg'
		call run 'bin:make TARGET='target '"RAMEXP='ramexp'" "CABLE='cable'" prserver.prg'
		call run 'cp -p prserver.prg 'machdir'/'cable'/pr'filext'.prg'
	    end
	end
    end

    /* Move the C128 boot sectors to the correct place. */
    call run 'mv bootsect.std bootsect.pia 'bindir'/c128'

    /* Make and move the extra C64 programs to the correct place. */
    
    call run 'rm -f *.prg'
    call run 'bin:make TARGET=c64 prutils'
    call run 'mv *.prg 'bindir'/c64'

    /* Remove some junk (cable-dependent, incorrectly named) */

    call run 'rm -f 'bindir'/*/prserver.prg'

end

/* Make the Amiga programs */

if buildamiga then do

    call run 'mkdir -p 'bindir'/amiga'
    call run 'bin:make CLIENT=AmigaDOS clean prmain'
    call run 'cp -p prmain 'bindir'/amiga/prmain'
    call run 'bin:make reallyclean'

end

/* Copy necessary libraries for Amiga programs */

if copylibs then do

    call run 'mkdir -p 'bindir'/amiga/libs'
    call run 'cp -p /Local/libs/ixemul.library.000 'bindir'/amiga/libs'

end

exit 0

run:
    say arg(1)
    rc = 0
    if doit then address command arg(1)
    if rc = 0 then return rc
    say "*** Build aborted."
    exit rc

usage:
    say "Usage: rx MakeAll.rexx [no8bit] [noamiga] [nolibs] >script"
    say "       execute script"
    say "if you wish to check the actions to be executed,"
    say "or     rx MakeAll.rexx doit [no8bit] [noamiga] [nolibs]"
    say "to run them immediately."
    exit 1
