#!/bin/sh
# Make all combinations of binaries
# Assumes GCC, GNU Make and DASM 2.02 with Olaf Seibert's modifications.

build8bit=TRUE
buildlinux=TRUE

while [ $# -gt 0 ]
do
	case "$1" in
	except8bit)
		build8bit=FALSE
		;;
	exceptlinux)
		buildlinux=FALSE
		;;
	*)
		echo "$0: unrecognized option '$1'."
	esac
	shift
done

BINDIR=../bin

### First make all 8-bitter .prgs:

if [ $build8bit = TRUE ]
then
	rm -f *.prg

	for target in pet3001 pet4001 vic20 c64 c128
	do
		# First make all binaries for the prlink cable:

		mkdir -p $BINDIR/$target/prlink
		make TARGET=$target RAMEXP=none CABLE=prlink \
		     clean prprgs || exit 1
		mv prserver.prg $BINDIR/$target/prlink
		mv *.prg $BINDIR/$target

		# Then make prserver binaries for the rest of the cables
		# except transnib and prlink88, which are Amiga specific:

		for cable in pc64 c64net # transnib prlink88
		do
			make TARGET=$target RAMEXP=none CABLE=$cable \
			     prserver.prg || exit 1
			mkdir $BINDIR/$target/$cable
			mv prserver.prg $BINDIR/$target/$cable
		done
	done

	# Move the C128 boot sectors to the correct place.
	mv bootsect.std bootsect.pia $BINDIR/c128/

	# The IEC cable is a special case for the VIC-20 and above.
	# The C16 currently only has the IEC cable option.

	mkdir -p $BINDIR/c16/iec
	make TARGET=c16 RAMEXP=none CABLE=iec prprgs || exit 1
	mv prserver.prg $BINDIR/c16/iec
	mv *.prg $BINDIR/c16

	for target in vic20 c64 c128
	do
		for cable in kontros iec
		do
			make TARGET=$target RAMEXP=none CABLE=$cable \
			     prserver.prg || exit 1
			mkdir $BINDIR/$target/$cable
			mv prserver.prg $BINDIR/$target/$cable
		done
	done

	# Make binaries with memory expansion support

	for target in c64 c128
	do
		for exp in pia reu piareu
		do
			if [ "$exp" = "piareu" ]; then
				expopt="piaexp | reuexp"
			else
				expopt=$exp"exp"
			fi

			for cable in kontros prlink pc64 \
				     c64net iec # transnib prlink88
			do
				make TARGET=$target RAMEXP="$expopt" \
				     CABLE=$cable prserver.prg || exit 1
				mv prserver.prg \
				   $BINDIR/$target/$cable/pr$exp.prg
			done
		done
	done
fi

if [ $buildlinux = TRUE ]
then
	rm -f prmain
	make CLIENT=Linux clean prmain || exit 1
	mkdir $BINDIR/linux
	mv prmain $BINDIR/linux/prmain && make reallyclean
fi
