/*
 * Copyright  1994-1996 Marko Mkel and Olaf Seibert
 * Original Linux and Commodore 64/128/Vic-20 version by Marko Mkel
 * Ported to the PET and the Amiga series by Olaf Seibert
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __linux__
#include <signal.h>
#include <unistd.h>
#include <sys/io.h>
#endif /* __linux__ */

#ifdef __FreeBSD__
#include <signal.h>
#include <unistd.h>
#include <machine/cpufunc.h>
#endif /* __FreeBSD__ */

#define __PRTRANS1_C__
#include "prtrans.h"

#ifndef DISK64E
/* X1541 cable constants */
#  define CLK 2
#  define DATA 8
#else
/* Disk64e cable constants */
#  define CLK 1
#  define DATA 2
#endif

static void cleanup (int i);

static void cleanup (int i) {
  outb(0, ctrladdr); /* this ensures that the IEC bus doesn't remain blocked */
  exit(-1);
}

int prinit (void) {
  ctrladdr = baseaddr + 2;

#ifdef __linux__
  if (ioperm (ctrladdr, 1, 1))
    return -1;

  seteuid(getuid()); /* give up the super user privileges */
  signal(SIGINT, cleanup);
  signal(SIGTERM, cleanup);
#endif /* __linux__ */

  return 0;
}

void prclose (void) {
  if (ctrladdr)
    outb(0, ctrladdr); /* release the serial lines */
}

void output (unsigned char byte) {
  psend (&byte, 1);
}

#ifndef MSDOS
void wait_output (unsigned char byte) {
  int i;

  /* send the first bit and wait for acknowledgement */

  /* drop CLK or DATA, depending on the bit value */
  outb((byte & 1) ? DATA : CLK, ctrladdr);

  /* wait for CLK==DATA==low */
  while ((~inb(ctrladdr)) & (CLK | DATA))
    usleep (SLEEP_TIME);

  for (i = 8; i--; byte >>= 1) {
    /* drop CLK or DATA, depending on the bit value */
    outb((byte & 1) ? DATA : CLK, ctrladdr);

    /* wait for CLK==DATA==low */
    while ((~inb(ctrladdr)) & (CLK | DATA));

    /* raise CLK and DATA */
    outb(0, ctrladdr);

    /* wait for CLK==DATA==high */
    while (inb(ctrladdr) & (CLK | DATA));
  }
}
#endif

unsigned input (void) {
  unsigned char byte;

  preceive (&byte, 1);
  return byte;
}

#ifndef MSDOS
unsigned wait_input (void) {
  int i;
  register unsigned char data = 0, regval;

  /* wait for the first bit */
  while (!(inb(ctrladdr) & (CLK | DATA)))
    usleep (SLEEP_TIME);

  for (i = 8; i--; ) {
    /* wait for CLK==low or DATA==low */
    while (!((regval = inb(ctrladdr)) & (CLK | DATA)));

    /* send an acknowledgement */
    outb(regval ^ (CLK | DATA), ctrladdr);

    /* read the data */
    data >>= 1;
    if (regval & DATA) data |= 128;

    /* wait for acknowledgement (CLK==high or DATA==high) */
    while (!((~inb(ctrladdr)) & (CLK | DATA)));

    /* raise CLK and DATA */
    outb(0, ctrladdr);
  }

  return data;
}
#endif

void send_switch () {
  do {
    /* drop CLK and DATA */
    outb(CLK | DATA, ctrladdr);
    usleep(SLEEP_TIME);
    /* raise CLK and DATA */
    outb(0, ctrladdr);
  }
  while (!(inb(ctrladdr) & CLK)); /* repeat until CLK==low */

  /* drop CLK */
  outb(CLK, ctrladdr);

  /* wait for DATA==low */
  while (!(inb(ctrladdr) & DATA));

  /* raise CLK */
  outb(0, ctrladdr);

  /* wait for DATA==high */
  while (inb(ctrladdr) & DATA);
}

void receive_switch () {
  /* wait for CLK==DATA==low */
  while ((~inb(ctrladdr)) & (CLK | DATA))
    usleep(SLEEP_TIME);

  do {
    /* drop CLK */
    outb(CLK, ctrladdr);
    usleep(SLEEP_TIME);
    /* raise CLK */
    outb(0, ctrladdr);
  } /* repeat until CLK==low && DATA==high */
  while ((inb(ctrladdr) & (CLK | DATA)) != CLK);

  /* drop DATA */
  outb(DATA, ctrladdr);

  /* wait for CLK==high */
  while (inb(ctrladdr) & CLK);

  /* raise DATA */
  outb(0, ctrladdr);
}

#if defined(GAS_ENABLED)
void psend (unsigned char *buffer, unsigned length) {
  asm ("movw ctrladdr,%%dx
        movl %1,%%ecx
        movl %0,%%ebx
	xorb %%al,%%al

sloop:  movb (%%ebx),%%ah
        push %%ecx
        movl $8,%%ecx

bsloop: shrb $1,%%ah"
#ifdef DISK64E	/* CLK==1, DATA==2 */
"
	adcb $1,%%al"
#else	/* X1541: CLK==2, DATA==8 */
"
	adcb $7,%%al
	andb $10,%%al"
#endif
"
        outb %%al,%%dx
owait1: inb %%dx,%%al
	notb %%al"
#ifdef DISK64E	/* CLK==1, DATA==2 */
"
	andb $3,%%al"
#else	/* X1541: CLK==2, DATA==8 */
"
	andb $10,%%al"
#endif
"
	jnz owait1
        outb %%al,%%dx
owait2: inb %%dx,%%al"
#ifdef DISK64E	/* CLK==1, DATA==2 */
"
	andb $3,%%al"
#else	/* X1541: CLK==2, DATA==8 */
"
	andb $10,%%al"
#endif
"
        jnz owait2
        loop bsloop

        pop %%ecx
        inc %%ebx
        loop sloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void psend (unsigned char *buffer, unsigned len) {
  __asm {
    mov dx,ctrladdr
    mov cx,len
    xor al,al

    push ds
    lds bx,buffer
sloop:
    mov ah,[bx]
    push cx
    mov cx,8
bloop:
    shr ah,1
#ifdef DISK64E
    adc al,1	/* CLK==1, DATA==2 */
#else
    adc al,7	/* CLK==2, DATA==8 */
    and al,10
#endif
    out dx,al
owait1:
    in al,dx
    not al
    and al,CLK | DATA
    jz nowait1
    in al,0x60
    dec al      /* esc pressed? */
    jne owait1
    pop cx
    jmp abort
nowait1:
    out dx,al
owait2:
    in al,dx
    and al,CLK | DATA
    jz nowait2
    in al,0x60
    dec al      /* esc pressed? */
    jne owait2
    pop cx
    jmp abort
nowait2:
    loop bloop
    pop cx

    inc bx
    loop sloop

abort:
    pop ds
  }

  if (inb(0x60) == 1) cleanup(-1);
}
#else
void psend (unsigned char *buffer, unsigned length) {
  int i;
  register unsigned char byte;

  while (length--) {
    byte = *buffer++;

    for (i = 8; i--; byte >>= 1) {
      /* drop CLK or DATA, depending on the bit value */
      outb((byte & 1) ? DATA : CLK, ctrladdr);

      /* wait for CLK==DATA==low */
      while ((~inb(ctrladdr)) & (CLK | DATA));

      /* raise CLK and DATA */
      outb(0, ctrladdr);

      /* wait for CLK==DATA==high */
      while (inb(ctrladdr) & (CLK | DATA));
    }
  }
}
#endif

#if defined(GAS_ENABLED)
void preceive (unsigned char *buffer, unsigned length) {
  asm ("movw ctrladdr,%%dx
        movl %1,%%ecx
        movl %0,%%ebx

rloop:  push %%ecx
        movl $8,%%ecx
brloop: inb %%dx,%%al"
#ifdef DISK64E	/* CLK==1, DATA==2 */
"
        testb $3,%%al
        jz brloop

	xorb $3,%%al"
#else	/* X1541: CLK==2, DATA==8 */
"
	testb $10,%%al
        jz brloop

	xorb $10,%%al"
#endif
"
	outb %%al,%%dx
#ifndef DISK64E
	rorb $1,%%al
#endif
	rorw $1,%%ax

iwait2: inb %%dx,%%al
	notb %%al"
#ifdef DISK64E	/* CLK==1, DATA==2 */
"
	andb $3,%%al"
#else	/* X1541: CLK==2, DATA==8 */
"
	andb $10,%%al"
#endif
"
	jz iwait2
        xorb %%al,%%al
        outb %%al,%%dx
        loop brloop

        pop %%ecx
        movb %%ah,(%%ebx)
        inc %%ebx
        loop rloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void preceive (unsigned char *buffer, unsigned len) {
  __asm {
    mov dx,ctrladdr
    mov cx,len

    push ds
    lds bx,buffer

rloop:
    push cx
    mov cx,8
brloop:
    in al,dx
    test al,CLK | DATA
    jnz niwait1
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz brloop
    pop cx
    jmp abort
niwait1:
    xor al,CLK | DATA
    out dx,al
#ifndef DISK64E
    ror al,1
#endif
    ror ax,1

iwait2:
    in al,dx
    not al
    and al,CLK | DATA
    jnz niwait2
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz iwait2
    pop cx
    jmp abort
niwait2:
    xor al,al
    out dx,al
    loop brloop

    pop cx
    mov [bx],ah
    inc bx
    loop rloop
abort:
    pop ds
  }

  if (inb(0x60) == 1) cleanup(-1);
}
#else
void preceive (unsigned char *buffer, unsigned length) {
  int i;
  register unsigned char data = 0, regval;

  while (length--) {
    for (i = 8; i--; ) {
      /* wait for CLK==low or DATA==low */
      while (!((regval = inb(ctrladdr)) & (CLK | DATA)));

      /* send an acknowledgement */
      outb(regval ^ (CLK | DATA), ctrladdr);

      /* read the data */
      data >>= 1;
      if (regval & DATA) data |= 128;

      /* wait for acknowledgement (CLK==high or DATA==high) */
      while (!((~inb(ctrladdr)) & (CLK | DATA)));

      /* raise CLK and DATA */
      outb(0, ctrladdr);
    }

    *buffer++ = data;
  }
}
#endif
