/*
 * Copyright  1994-1996 Marko Mkel and Olaf Seibert
 * Original Linux and Commodore 64/128/Vic-20 version by Marko Mkel
 * Ported to the PET and the Amiga series by Olaf Seibert
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __linux__
#include <unistd.h>
#include <sys/io.h>
#endif /* __linux__ */

#ifdef __FreeBSD__
#include <unistd.h>
#include <machine/cpufunc.h>
#endif /* __FreeBSD__ */

#define __PRTRANS4_C__
#include "prtrans.h"

int prinit (void) {
#ifdef __linux__
  if (ioperm (baseaddr, 2, 1))
    return -1;

  seteuid(getuid()); /* give up the super user privileges */
#endif /* __linux__ */

  outb(128, baseaddr); /* set the -FLAG signal to high */
  stataddr = baseaddr + 1;

  return 0;
}

void prclose (void) {
}

void output (unsigned char byte) {
  psend (&byte, 1);
}

#ifndef MSDOS
void wait_output (unsigned char byte) {
  outb(byte & 0x0f, baseaddr);
  while (!(128 & inb(stataddr)))
    usleep (SLEEP_TIME); /* wait between each try */

  outb(128, baseaddr);
  outb(byte >> 4, baseaddr);
  while (128 & inb(stataddr));
  outb(128, baseaddr);
}
#endif /* !defined(MSDOS) */

unsigned input (void) {
  unsigned char byte;

  preceive (&byte, 1);
  return byte;
}

#ifndef MSDOS
unsigned wait_input (void) {
  register unsigned char data;

  outb(0, baseaddr);
  while (!(128 & inb(stataddr)))
    usleep (SLEEP_TIME); /* wait between each try */

  outb(128, baseaddr);
  data = (inb(stataddr) & 0x78) >> 3;
  outb(0, baseaddr);
  while (128 & inb(stataddr));
  data |= (inb(stataddr) & 0x78) << 1;
  outb(128, baseaddr);

  return data;
}
#endif /* !defined(MSDOS) */

void send_switch() {}
void receive_switch() {}

#if defined(GAS_ENABLED)
void psend (unsigned char *buffer, unsigned length) {
  asm ("movw baseaddr,%%dx
        movl %1,%%ecx
        movl %0,%%ebx

sloop:  movb (%%ebx),%%al
        andb $0x0f,%%al
        outb %%al,%%dx
        incw %%dx

ewait:  inb %%dx,%%al
        testb $0x80,%%al
        jz ewait

        decw %%dx
        movb $0x80,%%al
        outb %%al,%%dx
        movb (%%ebx),%%al
        shrb $4,%%al
        outb %%al,%%dx
        incw %%dx

zwait:  inb %%dx,%%al
        testb $0x80,%%al
        jnz zwait

        decw %%dx
        movb $0x80,%%al
        outb %%al,%%dx

        inc %%ebx
        loop sloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void psend (unsigned char *buffer, unsigned len) {
  __asm {
    push ds
    mov dx,baseaddr
    mov cx,len
    lds bx,buffer
sloop:
    mov al,[bx]
    and al,0x0f
    out dx,al
    inc dx

ewait:
    in al,dx
    test al,0x80
    jnz newait
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz ewait
    jz abort

newait:
    dec dx
    mov al,0x80
    out dx,al
    mov al,[bx]
# ifdef USE286
    shr al,4
# else
    shr al,1
    shr al,1
    shr al,1
    shr al,1
# endif
    out dx,al
    inc dx

zwait:
    in al,dx
    test al,0x80
    jz nzwait
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz zwait
    jz abort

nzwait:
    dec dx
    mov al,0x80
    out dx,al

    inc bx
    loop sloop
abort:
    pop ds
  }

  if (inb(0x60) == 1) exit(-1);
}
#else
void psend (unsigned char *buffer, unsigned length) {
  register unsigned char byte;

  while (length--) {
    byte = *buffer++;

    outb(byte & 0x0f, baseaddr);
    while (!(128 & inb(stataddr)));
    outb(128, baseaddr);
    outb(byte >> 4, baseaddr);
    while (128 & inb(stataddr));
    outb(128, baseaddr);
  }
}
#endif

#if defined(GAS_ENABLED)
void preceive (unsigned char *buffer, unsigned length) {
  asm ("movw baseaddr,%%dx
        movl %1,%%ecx
        movl %0,%%ebx

rloop:  xorb %%al,%%al
        outb %%al,%%dx
        incw %%dx

dwait:  inb %%dx,%%al
        testb $0x80,%%al        
        jz dwait

        andb $0x78,%%al
        shrb $3,%%al
        movb %%al,(%%ebx)
        decw %%dx
        movb $0x80,%%al
        outb %%al,%%dx
        xorb %%al,%%al
        outb %%al,%%dx
        incw %%dx

vwait:  inb %%dx,%%al
        testb $0x80,%%al
        jnz vwait

        andb $0x78,%%al
        shlb $1,%%al
        orb %%al,(%%ebx)
        decw %%dx
        movb $0x80,%%al
        outb %%al,%%dx

        inc %%ebx
        loop rloop"
       : /* no outputs */
       : "g" (buffer), "g" (length)
       : "ax", "ebx", "ecx", "dx");
}
#elif defined(MASM_ENABLED)
void preceive (unsigned char *buffer, unsigned len) {
  __asm {
    push ds
    mov dx,baseaddr
    mov cx,len
    lds bx,buffer

rloop:
    xor al,al
    out dx,al
    inc dx

dwait:
    in al,dx
    test al,0x80
    jnz ndwait
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz dwait
    jz abort

ndwait:
    and al,0x78
# ifdef USE286
    shr al,3
# else
    shr al,1
    shr al,1
    shr al,1
# endif
    mov [bx],al
    dec dx
    mov al,0x80
    out dx,al
    xor al,al
    out dx,al
    inc dx

vwait:
    in al,dx
    test al,0x80
    jz nvwait
    in al,0x60  ; is the esc key pressed?
    dec al
    jnz vwait
    jz abort

nvwait:
    and al,0x78
    shl al,1
    or [bx],al
    dec dx
    mov al,0x80
    out dx,al

    inc bx
    loop rloop

abort:
    pop ds
  }

  if (inb(0x60) == 1) exit(-1);
}
#else
void preceive (unsigned char *buffer, unsigned length) {
  register unsigned char data;

  while (length--) {
    outb(0, baseaddr);
    while (!(128 & inb(stataddr)));
    outb(128, baseaddr); 
    data = (inb(stataddr) & 0x78) >> 3;
    outb(0, baseaddr);
    while (128 & inb(stataddr));
    *buffer++ = data | ((inb(stataddr) & 0x78) << 1);  
    outb(128, baseaddr);
  }
}
#endif
